unit DirectSound8Capture;

interface

uses
	Windows, SysUtils, Classes, MMSystem, directx9, DSound;

type
	TDirectSoundCapture = class;

	//̃NXInitialize̎ɓnΊYfoCXŏo
	TDirectSoundDevice = class( TCollectionItem )
	private
		FDescription: string;
		FModuleName: string;
		FGUID: PGUID;
    	procedure SetGUID(const Value: PGUID);
	public
		destructor Destroy(); override;
		property GUID: PGUID read FGUID write SetGUID;
		property Description: string read FDescription write FDescription;
		property ModuleName: string read FModuleName write FModuleName;
	end;

	TDirectSoundDevices = class( TCollection )
	private
		function GetItem(Index: Integer): TDirectSoundDevice;
		procedure SetItem(Index: Integer; const Value: TDirectSoundDevice);
	public
		constructor Create();
		function Add: TDirectSoundDevice;
		function Insert(Index: Integer): TDirectSoundDevice;
		property Items[Index: Integer]: TDirectSoundDevice read GetItem write SetItem; default;
	end;

	//iSuspendŁA^̂Resume
	TDirectSoundCaptureNotifyThread = class( TThread )
	private
		FParent: TDirectSoundCapture;
	public
		constructor Create( const Parent: TDirectSoundCapture );
		procedure Execute(); override;
	end;

	EDirectSoundCapture = class( Exception );
	TDirectSoundCaptureEvent = procedure ( const Sender: TObject; const Offset: Cardinal; BufferSize: Cardinal ) of object;
	TDirectSoundCapture = class
	private
		FDevices: TDirectSoundDevices;
		FCapture: IDirectSoundCapture8;
		FCaptureBuffer: IDirectSoundCaptureBuffer8;
		FNotifies: array of THandle;
		FBlockSize: Cardinal;
		FNextBlock: Cardinal;
		FBufferSize, FBlockAlign: DWORD;
		FThread: TDirectSoundCaptureNotifyThread;
		FOnCaptured: TDirectSoundCaptureEvent;
		FCS: TRTLCriticalSection;
		FNotifyThreadPriority: TThreadPriority;
		function GetDeviceList: TDirectSoundDevices;
		procedure ThreadProc(); //͕ʃXbhĂ΂邼I
		function GetPosition: DWORD;
		procedure SetNotifyThreadPriority(const Value: TThreadPriority);
	public
		constructor Create();
		destructor Destroy(); override;

		//āA
		procedure Initialize( const Device: TDirectSoundDevice = nil );

		//obt@쐬āA
		procedure CreateCaptureBuffer( const Frequency, Bits, Channels: Integer; const LengthbyMSec: Cardinal; const NumEvents: Cardinal );

		//^Ď~߂āA
		procedure Start();
		procedure Stop();

		//obt@JāA
		procedure DestroyCaptureBuffer();

		//I
		procedure Finalize();

		procedure RemakeDeviceList();
		procedure BlockCopy( const Dest: Pointer; const Offset: Cardinal; const Length: Cardinal ); //ubN]
		property DeviceList: TDirectSoundDevices read GetDeviceList;
		property OnCaptured: TDirectSoundCaptureEvent read FOnCaptured write FOnCaptured;

		property Capture: IDirectSoundCapture8 read FCapture;
		property Buffer: IDirectSoundCaptureBuffer8 read FCaptureBuffer;
		property BufferLength: DWORD read FBufferSize;
		property Position: DWORD read GetPosition;
		property NotifyThreadPriority: TThreadPriority read FNotifyThreadPriority write SetNotifyThreadPriority;
	end;

implementation

{ TDirectSoundDevice }

destructor TDirectSoundDevice.Destroy;
begin
	if Assigned( FGUID ) then Dispose( FGUID );
	FGUID := nil;

	inherited;
end;

procedure TDirectSoundDevice.SetGUID(const Value: PGUID);
begin
	if Value = nil then begin
		if Assigned( FGUID ) then Dispose( FGUID );
		FGUID := nil;
	end else begin
		if not Assigned( FGUID ) then New( FGUID );
		FGUID^ := Value^;
	end;
end;

{ TDirectSoundDevices }

function TDirectSoundDevices.Add: TDirectSoundDevice;
begin
	Result := TDirectSoundDevice( inherited Add() );
end;

constructor TDirectSoundDevices.Create;
begin
	inherited Create( TDirectSoundDevice );
end;

function TDirectSoundDevices.GetItem(Index: Integer): TDirectSoundDevice;
begin
	Result := TDirectSoundDevice( inherited GetItem( Index ) );
end;

function TDirectSoundDevices.Insert(Index: Integer): TDirectSoundDevice;
begin
	Result := TDirectSoundDevice( inherited Insert( Index ) );
end;

procedure TDirectSoundDevices.SetItem(Index: Integer; const Value: TDirectSoundDevice);
begin
	inherited SetItem( Index, Value );
end;

{ TDirectSoundCapture }

procedure TDirectSoundCapture.BlockCopy(const Dest: Pointer; const Offset, Length: Cardinal);
var
	Len1, Len2: DWORD;
	p1, p2: Pointer;
	p: PByte;
	hr: HRESULT;
begin
	p := Dest;
	EnterCriticalSection( FCS );
	try
		if not Assigned( FCaptureBuffer ) then
			raise EDirectSoundCapture.Create( 'obt@쐬Ă܂B' );

		hr := FCaptureBuffer.Lock( Offset, Length, p1, @Len1, p2, @Len2, 0 );
		if hr <> DS_OK then
			raise EDirectSoundCapture.Create( 'obt@̃bNɎs܂B' );
		try
			Move( p1^, p^, Len1 );
			if Len1 < Length then begin
				Inc( p, Len1 );
				Move( p2^, p^, Len2 );
			end;
		finally
			FCaptureBuffer.Unlock( p1, Len1, p2, Len2 );
			//܂͎sǂ悤ȂƂc
		end;
	finally
		LeaveCriticalSection( FCS );
	end;
end;

constructor TDirectSoundCapture.Create;
begin
	inherited;

	InitializeCriticalSection( FCS );

	FNotifyThreadPriority := tpNormal;
	FThread := nil;
	FCapture := nil;
	FDevices := TDirectSoundDevices.Create();
end;

procedure TDirectSoundCapture.CreateCaptureBuffer(const Frequency, Bits, Channels: Integer; const LengthbyMSec: Cardinal; const NumEvents: Cardinal );
var
	d: Cardinal;
	i: Integer;
	hr: HRESULT;
	BufDesc: DSCBUFFERDESC;
	wfex: TWaveFormatEx;
	NotifyIntf: IDirectSoundNotify;
	NotifyPos: array of DSBPOSITIONNOTIFY;
begin
	if not Assigned( FCapture ) then
		raise EDirectSoundCapture.Create( 'foCXĂ܂B' );

	if Assigned( FCaptureBuffer ) then
		raise EDirectSoundCapture.Create( 'obt@͊ɍ쐬Ă܂B' );

	FBlockAlign := Channels * Bits div 8;
	with wfex do begin
		wFormatTag := WAVE_FORMAT_PCM;
		nChannels := Channels;
		nSamplesPerSec := Frequency;
		nBlockAlign := FBlockAlign;
		nAvgBytesPerSec := Frequency * nBlockAlign;
		wBitsPerSample := Bits;
		cbSize := 0;
	end;

	d := Trunc( LengthbyMSec/1000*wfex.nAvgBytesPerSec ) div FBlockAlign;
	FBufferSize := d * FBlockAlign; //ubNEɃACgꂽobt@TCY
	with BufDesc do begin
		dwSize := sizeof( DSCBUFFERDESC );
		dwFlags := 0;
		dwBufferBytes := FBufferSize;
		dwReserved := 0;
		lpwfxFormat := @wfex;
		dwFXCount := 0;
		lpDSCFXDesc := nil;
	end;

	hr := FCapture.CreateCaptureBuffer( @BufDesc, IDirectSoundCaptureBuffer(FCaptureBuffer), nil );
	if hr <> DS_OK then
		raise EDirectSoundCapture.CreateFmt( '%dHz %dBit %dch ł̃obt@̍쐬Ɏs܂B', [ Frequency, Bits, Channels ] );

	//ʒmpCxg\iʉ߃Cxg{Xgbvʒmj
	SetLength( FNotifies, NumEvents+1 );
	SetLength( NotifyPos, NumEvents+1 );

	//ʉ߃Cxg
	FBlockSize := ( d div NumEvents )*FBlockAlign; //ubNEŃL`ƃACgĂ
	for i := 0 to NumEvents-1 do begin
		FNotifies[i] := CreateEvent( nil, FALSE, FALSE, nil );
		with NotifyPos[i] do begin
			dwOffset := FBlockSize*Cardinal(i);
			hEventNotify := FNotifies[i];
		end;
	end;

	//Xgbvʒm
	FNotifies[NumEvents] := CreateEvent( nil, TRUE, FALSE, nil );
	with NotifyPos[NumEvents] do begin
		dwOffset := DSBPN_OFFSETSTOP;
		hEventNotify := FNotifies[NumEvents];
	end;

	hr := FCaptureBuffer.QueryInterface( IID_IDirectSoundNotify, NotifyIntf );
	if hr <> S_OK then
		raise EDirectSoundCapture.Create( 'IDirectSoundNotify ̎擾Ɏs܂B' );
	NotifyIntf.SetNotificationPositions( Length( NotifyPos ), @NotifyPos[0] );
	NotifyIntf := nil;

	FThread := TDirectSoundCaptureNotifyThread.Create( Self );
	FThread.Priority := FNotifyThreadPriority;
end;

destructor TDirectSoundCapture.Destroy;
begin
	if Assigned( FCapture ) then Finalize();

	FDevices.Free;

	DeleteCriticalSection( FCS );

	inherited;
end;

procedure TDirectSoundCapture.DestroyCaptureBuffer;
var
	i: Integer;
begin
	Stop();

	//XbhJ
	FThread.Terminate();
	SetEvent( FNotifies[ High( FNotifies ) ] );
	FThread.Resume();
	FThread.WaitFor();
	FreeAndNil(	FThread );

	FCaptureBuffer := nil;

	//ʒmpCxgJ
	for i := Low( FNotifies ) to High( FNotifies ) do
		CloseHandle( FNotifies[i] );
end;

procedure TDirectSoundCapture.Finalize;
begin
	if Assigned( FCaptureBuffer ) then DestroyCaptureBuffer();
	FCapture := nil;
end;

function TDirectSoundCapture.GetDeviceList: TDirectSoundDevices;
begin
	if FDevices.Count = 0 then RemakeDeviceList();
	Result := FDevices;
end;

function TDirectSoundCapture.GetPosition: DWORD;
var
	hr: HRESULT;
begin
	EnterCriticalSection( FCS );
	try
		if not Assigned( FCaptureBuffer ) then
			raise EDirectSoundCapture.Create( 'obt@쐬Ă܂B' );

		hr := FCaptureBuffer.GetCurrentPosition( nil, @Result );
		if hr <> DS_OK then
			raise EDirectSoundCapture.Create( 'ʒu̎擾Ɏs܂B' );
	finally
		LeaveCriticalSection( FCS );
	end;
end;

procedure TDirectSoundCapture.Initialize( const Device: TDirectSoundDevice );
var
	hr: HRESULT;
begin
	if Assigned( FCapture ) then Finalize();

	if Device = nil then
		hr := DirectSoundCaptureCreate8( nil, FCapture, nil )
	else
		hr := DirectSoundCaptureCreate8( Device.GUID, FCapture, nil );
	if hr <> S_OK then
		raise EDirectSoundCapture.Create( 'IDirectSoundCapture8 ̎擾Ɏs܂B' );
end;

procedure TDirectSoundCapture.RemakeDeviceList;
	function EnumerateDevicesCallback(Para0: PGUID; Para1: LPCSTR; Para2: LPCSTR; Para3: Pointer): BOOL; stdcall;
	var
		Devices: TDirectSoundDevices absolute Para3;
		d: TDirectSoundDevice;
	begin
		d := Devices.Add();
		if Assigned( Para0 ) then d.GUID := Para0;
		d.Description := Para1;
		d.ModuleName := Para2;
		Result := TRUE;
	end;
begin
	FDevices.Clear();
	DirectSoundCaptureEnumerate( @EnumerateDevicesCallback, FDevices );
end;

procedure TDirectSoundCapture.SetNotifyThreadPriority(
  const Value: TThreadPriority);
begin
	FNotifyThreadPriority := Value;
	if Assigned( FThread ) then
		FThread.Priority := FNotifyThreadPriority;
end;

procedure TDirectSoundCapture.Start;
var
	hr: HRESULT;
begin
	EnterCriticalSection( FCS );
	try
		if not Assigned( FCaptureBuffer ) then
			raise EDirectSoundCapture.Create( 'obt@쐬Ă܂B' );

		FNextBlock := 0;
		FThread.Resume();
		hr := FCaptureBuffer.Start( DSCBSTART_LOOPING );
		if hr <> DS_OK then
			raise EDirectSoundCapture.Create( 'ĐɎs܂B' );
	finally
		LeaveCriticalSection( FCS );
	end;
end;

procedure TDirectSoundCapture.Stop;
var
	hr: HRESULT;
begin
	EnterCriticalSection( FCS );
	try
		if not Assigned( FCaptureBuffer ) then
			raise EDirectSoundCapture.Create( 'obt@쐬Ă܂B' );

		hr := FCaptureBuffer.Stop();
		if hr <> DS_OK then
			raise EDirectSoundCapture.Create( '~Ɏs܂B' );
	finally
		LeaveCriticalSection( FCS );
	end;
end;

procedure TDirectSoundCapture.ThreadProc;
var
	Suspend: Boolean;
	Event, CapturePos, ReadPos, st, ed, Blocks, n: DWORD;
begin
	while True do begin
		Suspend := False;
		//ł͑SẴCxgIuWFNgĎ邽߂WaitForMultipleObjectsgp邪A
		//̊֐ł͕̃CxgɓBꍇos\Ȃ߈ȌʂɃ`FbNB
		//炭CfbNX̂̂D悳ƎvA̎dlSDKɋLڂĂ炸A
		//dlύXĂȂ悤ɓo^ɈˑȂ`Ŏ
		EnterCriticalSection( FCS );
		try
			if FThread.Terminated then break;
		finally
			LeaveCriticalSection( FCS );
		end;
		Event := WaitForMultipleObjects( Length( FNotifies ), @FNotifies[0], FALSE, INFINITE );
		EnterCriticalSection( FCS );
		try
			if FThread.Terminated then break;
		finally
			LeaveCriticalSection( FCS );
		end;
		if Event = WAIT_FAILED then break;

		//~CxgĂ
		if ( WaitForSingleObject( FNotifies[High(FNotifies)], 0 ) = WAIT_OBJECT_0 ) then begin
			EnterCriticalSection( FCS );
			try
				FCaptureBuffer.GetCurrentPosition( @CapturePos, @ReadPos );
				if CapturePos < ReadPos then Inc( CapturePos, FBufferSize );

				st := FBlockSize*FNextBlock; //Lv`̎n_
				ed := ReadPos; //ǂݍ݃J[\ʒuI_
				if st > ed then Inc( ed, FBufferSize );

				if Assigned( FOnCaptured ) then
					FOnCaptured( Self, st, ed - st );

				ResetEvent( FNotifies[High(FNotifies)] );
				Suspend := True;
			finally
				LeaveCriticalSection( FCS );
			end;
		//ʂ̃CxgĂ
		end else begin
			EnterCriticalSection( FCS );
			try
				Blocks := High(FNotifies); //ʉ߃Cxǧ
				st := FBlockSize*FNextBlock; //Jnn_
				FCaptureBuffer.GetCurrentPosition( @CapturePos, @ReadPos );
				if CapturePos < ReadPos then Inc( CapturePos, FBufferSize );

				//̈ʒuǂővH
				while ( st > ReadPos )and( st <= CapturePos ) do begin
					Inc( st, FBlockSize );
					Inc( FNextBlock );
					if FNextBlock >= Blocks then Dec( FNextBlock, Blocks );
				end;

				n := 0;
				ed := st;
				//LȃubŇTo
				if ed > ReadPos then Inc( ReadPos, FBufferSize );
				while ed+FBlockSize <= ReadPos do begin
					Inc( n );
					Inc( ed, FBlockSize );
				end;

				//LȃubN
				if n <> 0 then begin
					if Assigned( FOnCaptured ) then
						FOnCaptured( Self, FNextBlock*FBlockSize, n*FBlockSize );
					Inc( FNextBlock, n );
					while FNextBlock >= Blocks do Dec( FNextBlock, Blocks );
				end;
			finally
				LeaveCriticalSection( FCS );
			end;
		end;

		if Suspend then begin
			EnterCriticalSection( FCS );
			try
				if FThread.Terminated then break;
			finally
				LeaveCriticalSection( FCS );
			end;
			FThread.Suspend();
		end;
	end;
end;

{ TDirectSoundCaptureNotifyThread }

constructor TDirectSoundCaptureNotifyThread.Create(const Parent: TDirectSoundCapture);
begin
	inherited Create( True );
	FParent := Parent;
end;

procedure TDirectSoundCaptureNotifyThread.Execute;
begin
	FParent.ThreadProc();
end;

end.
