unit OggVorbisEncoder;

interface

uses
	Windows, SysUtils, Classes, OggVorbisHeader, OggVorbisPacket;

type
	TOggVorbisEncoder = class;

	TDataEncodeThread = class( TThread )
	private
		FParent: TOggVorbisEncoder;
	public
		constructor Create( const Parent: TOggVorbisEncoder );
		destructor Destroy(); override;
		procedure Execute(); override;
	end;

	TOggVorbisEncoder = class
	private
		FSourceChain: TOggVorbisEncoder_PacketDataList;
		FDestChain: TOggVorbisEncoder_PacketDataList;
		FCS: TRTLCriticalSection;
		FVorbisInfo: TVorbisInfo;
		FVorbisComment: TVorbisComment;
		FVorbisDSPState: TVorbisDSPState;
		FVorbisBlock: TVorbisBlock;
		FOggStreamState: TOggStreamState;
		FOggPage: TOggPage;
		FOggPacket: TOggPacket;
		FHeader: TMemoryStream;
		FThread: TDataEncodeThread;
		FBits: Integer;
		FEOS: Boolean;
	FEncodeThreadPriority: TThreadPriority;
		procedure FlushBuffer();
		//ʃXbhŎsI܂̂TrueԂ
		function Encode(): Boolean;
		procedure SetEncodeThreadPriority(const Value: TThreadPriority);
	public
		constructor Create();
		destructor Destroy(); override;
		procedure BeginEncode( const Channels: Integer; const Rate: Integer; const Bits: Integer; const BaseQuality: Single );
		procedure EndEncode();
		procedure AddPCM( const Data: Pointer; const Length: Cardinal ); overload;
		procedure AddPCM( const Data1: Pointer; const Length1: Cardinal; const Data2: Pointer; const Length2: Cardinal ); overload;
		function SrcBufferCount(): Cardinal;
		function DestBufferCount(): Cardinal;
		procedure WriteHeaderToStream( const Stream: TStream );
		procedure WriteAllBodyToStream( const Stream: TStream );
		procedure WriteOldestBodyAndFreeToStream( const Stream: TStream );
		procedure WriteAllBodyAndFreeToStream( const Stream: TStream );
		function IsOggPacketAvailable(): Boolean;
		property EncodeThreadPriority: TThreadPriority read FEncodeThreadPriority write SetEncodeThreadPriority;
	end;

implementation

var
	ovlib: TOggVorbisEncodeFunctions;

{ TOggVorbisEncoder }

procedure TOggVorbisEncoder.AddPCM(const Data: Pointer; const Length: Cardinal);
var
	pd: TOggVorbisEncoder_PacketData;
begin
	pd := TOggVorbisEncoder_PacketData.Create( Data, Length );

	EnterCriticalSection( FCS );
	try
		FSourceChain.InsertLast( pd );
	finally
		LeaveCriticalSection( FCS );
	end;
	FThread.Resume();
end;

procedure TOggVorbisEncoder.AddPCM(const Data1: Pointer;
  const Length1: Cardinal; const Data2: Pointer; const Length2: Cardinal);
var
	pd: TOggVorbisEncoder_PacketData;
begin
	pd := TOggVorbisEncoder_PacketData.Create( Data1, Length1, Data2, Length2 );

	EnterCriticalSection( FCS );
	try
		FSourceChain.InsertLast( pd );
	finally
		LeaveCriticalSection( FCS );
	end;
	FThread.Resume();
end;

procedure TOggVorbisEncoder.BeginEncode(const Channels, Rate, Bits: Integer; const BaseQuality: Single);
var
	header, header_comm, header_code: TOggPacket;
	pd: TOggVorbisEncoder_PacketData;
begin
	//݃GR[h
	if Assigned( FThread ) then
		raise Exception.Create('GR[hɍēxGR[hJn悤Ƃ');

	ovlib.vorbis_info_init( @FVorbisInfo );

	if ovlib.vorbis_encode_init_vbr( @FVorbisInfo, Channels, Rate, BaseQuality )<>0 then
		raise Exception.Create('VBR[hł̏s');

	FBits := Bits; //_Ci~bNȕύX͕s\

	ovlib.vorbis_comment_init( @FVorbisComment );
	ovlib.vorbis_analysis_init( @FVorbisDSPState, @FVorbisInfo );
	ovlib.vorbis_block_init( @FVorbisDSPState, @FVorbisBlock );

	ovlib.ogg_stream_init( @FOggStreamState, GetTickCount() );

	ovlib.vorbis_analysis_headerout( @FVorbisDSPState, @FVorbisComment, @header, @header_comm, @header_code );
	ovlib.ogg_stream_packetin( @FOggStreamState, @header );
	ovlib.ogg_stream_packetin( @FOggStreamState, @header_comm );
	ovlib.ogg_stream_packetin( @FOggStreamState, @header_code );

	FEOS := False;

	while not FEOS do begin
		if ovlib.ogg_stream_flush( @FOggStreamState, @FOggPage ) = 0 then break;
		FlushBuffer();
	end;

	//Ƃŉxg܂킷̂Ńwb_͕ʂɉĂ
	EnterCriticalSection( FCS );
	try
		while FDestChain.Count<>0 do begin
			pd := TOggVorbisEncoder_PacketData( FDestChain.First );
			FHeader.WriteBuffer( pd.Data^, pd.Length );
			pd.Free;
		end;
	finally
		LeaveCriticalSection( FCS );
	end;

	FThread := TDataEncodeThread.Create( Self );
	FThread.Priority := FEncodeThreadPriority;
end;

procedure TOggVorbisEncoder.EndEncode;
begin
	AddPCM( nil, 0 );
	FThread.Terminate();
	FThread.Resume();
	FThread.WaitFor();
	FreeAndNil( FThread );

	ovlib.ogg_stream_clear( @FOggStreamState );
	ovlib.vorbis_block_clear( @FVorbisBlock );
	ovlib.vorbis_dsp_clear( @FVorbisDSPState );
	ovlib.vorbis_comment_clear( @FVorbisComment );

	ovlib.vorbis_info_clear( @FVorbisInfo );

	FHeader.Clear();
	FSourceChain.Clear();
end;

constructor TOggVorbisEncoder.Create;
begin
	if not Assigned( ovlib ) then
		raise Exception.Create('ɕKv DLLiogg.dll, vorbis.dll, vorbisenc.dllj܂B');

	inherited;

	InitializeCriticalSection( FCS );
	FHeader := TMemoryStream.Create();
	FSourceChain := TOggVorbisEncoder_PacketDataList.Create();
	FDestChain := TOggVorbisEncoder_PacketDataList.Create();

	FThread := nil;
	FEncodeThreadPriority := tpNormal;

	FillChar( FVorbisInfo, 0, sizeof( FVorbisInfo ) );
	FillChar( FOggStreamState, 0, sizeof( FOggStreamState ) );
	FillChar( FVorbisComment, 0, sizeof( FVorbisComment ) );
	FillChar( FVorbisDSPState, 0, sizeof( FVorbisDSPState ) );
	FillChar( FVorbisBlock, 0, sizeof( FVorbisBlock ) );
	FillChar( FOggPage, 0, sizeof( FOggPage ) );
	FillChar( FOggPacket, 0, sizeof( FOggPacket ) );
end;

function TOggVorbisEncoder.DestBufferCount: Cardinal;
begin
	EnterCriticalSection( FCS );
	try
		Result := FDestChain.Count;
	finally
		LeaveCriticalSection( FCS );
	end;
end;

destructor TOggVorbisEncoder.Destroy;
begin
	if Assigned( FThread ) then EndEncode();

	if Assigned( FDestChain ) then FreeAndNil( FDestChain );
	if Assigned( FSourceChain ) then FreeAndNil( FSourceChain );
	if Assigned( FHeader ) then FreeAndNil( FHeader );
	DeleteCriticalSection( FCS );

	inherited;
end;

function TOggVorbisEncoder.Encode(): Boolean;
var
	pd: TOggVorbisEncoder_PacketData;
	Samples, i, ch, ct: Integer;
	pfInputBuffer: PPSingleArrayArray;
	pi8: PShortint;
	pi16: PSmallint;
	pi32: PLongint;
	//gȂǑΉo邩mȂ
//	pi64: PInt64;
//	pf32: PSingle;
//	pf64: PDouble;
begin
	pd := nil;
	EnterCriticalSection( FCS );
	try
		ct := FSourceChain.Count;
		if ct <> 0 then begin
			pd := TOggVorbisEncoder_PacketData( FSourceChain.First );
			pd.Extract;
			Dec( ct );
		end;
	finally
		LeaveCriticalSection( FCS );
	end;

	if Assigned( pd ) then begin
		Samples := pd.Length div ( Cardinal(FBits) div 8 ) div Cardinal(FVorbisInfo.Channels);
		pfInputBuffer := ovlib.vorbis_analysis_buffer( @FVorbisDSPState, Samples );
		case FBits div 8 of
			1: begin //8bit integer
				pi8 := pd.Data;
				for i := 0 to Samples-1 do begin
					for ch := 0 to FVorbisInfo.Channels-1 do begin
						pfInputBuffer[ch][i] := pi8^/128.0;
						Inc( pi8 );
					end;
				end;
			end;
			2: begin //16bit integer
				pi16 := pd.Data;
				for i := 0 to Samples-1 do begin
					for ch := 0 to FVorbisInfo.Channels-1 do begin
						pfInputBuffer[ch][i] := pi16^/32768.0;
						Inc( pi16 );
					end;
				end;
			end;
			4: begin //32bit integer
				pi32 := pd.Data;
				for i := 0 to Samples-1 do begin
					for ch := 0 to FVorbisInfo.Channels-1 do begin
						pfInputBuffer[ch][i] := pi32^/2147483648.0;
						Inc( pi32 );
					end;
				end;
			end;
		end;

		pd.Free;

		ovlib.vorbis_analysis_wrote( @FVorbisDSPState, Samples );
		while ovlib.vorbis_analysis_blockout( @FVorbisDSPState, @FVorbisBlock ) = 1 do begin
				ovlib.vorbis_analysis( @FVorbisBlock, nil );
			ovlib.vorbis_bitrate_addblock( @FVorbisBlock );
				while ovlib.vorbis_bitrate_flushpacket( @FVorbisDSPState, @FOggPacket )<>0 do begin
				ovlib.ogg_stream_packetin( @FOggStreamState, @FOggPacket );
				while True do begin
					if ovlib.ogg_stream_pageout( @FOggStreamState, @FOggPage ) = 0 then break;
					FlushBuffer();
					if ( ovlib.ogg_page_eos( @FOggPage )<>0 ) then FEOS := True;
				end;
			end;
		end;

	end;
	Result := ct<>0;
end;

procedure TOggVorbisEncoder.FlushBuffer;
var
	pd: TOggVorbisEncoder_PacketData;
begin
	if not FEOS then begin
		pd := TOggVorbisEncoder_PacketData.Create( FOggPage.Header, FOggPage.HeaderLen, FOggPage.Body, FOggPage.BodyLen );
		EnterCriticalSection( FCS );
		try
			FDestChain.InsertLast( pd );
		finally
			LeaveCriticalSection( FCS );
		end;
	end;
end;

function TOggVorbisEncoder.IsOggPacketAvailable: Boolean;
begin
	EnterCriticalSection( FCS );
	try
		Result := FDestChain.Count <> 0;
	finally
		LeaveCriticalSection( FCS );
	end;
end;

procedure TOggVorbisEncoder.SetEncodeThreadPriority( const Value: TThreadPriority );
begin
	FEncodeThreadPriority := Value;
	if Assigned( FThread ) then
		FThread.Priority := FEncodeThreadPriority;
end;

function TOggVorbisEncoder.SrcBufferCount: Cardinal;
begin
	EnterCriticalSection( FCS );
	try
		Result := FSourceChain.Count;
	finally
		LeaveCriticalSection( FCS );
	end;
end;

procedure TOggVorbisEncoder.WriteAllBodyAndFreeToStream( const Stream: TStream );
var
	pd: TOggVorbisEncoder_PacketData;
begin
	while True do begin
		EnterCriticalSection( FCS );
		try
			pd := TOggVorbisEncoder_PacketData( FDestChain.First );
			if not Assigned( pd ) then break;
			pd.Extract();
		finally
			LeaveCriticalSection( FCS );
		end;
		try
			Stream.WriteBuffer( pd.Data^, pd.Length );
		finally
			pd.Free;
		end;
	end;
end;

procedure TOggVorbisEncoder.WriteAllBodyToStream(const Stream: TStream);
var
	pd: TOggVorbisEncoder_PacketData;
begin
	EnterCriticalSection( FCS );
	try
		pd := TOggVorbisEncoder_PacketData( FDestChain.First );
		if not Assigned( pd ) then Exit;
		while Assigned( pd.Next ) do begin
			Stream.WriteBuffer( pd.Data^, pd.Length );
			pd := TOggVorbisEncoder_PacketData( pd.Next );
		end;
	finally
		LeaveCriticalSection( FCS );
	end;
end;

procedure TOggVorbisEncoder.WriteHeaderToStream(const Stream: TStream);
begin
	EnterCriticalSection( FCS );
	try
		FHeader.Position := 0;
		Stream.CopyFrom( FHeader, FHeader.Size );
	finally
		LeaveCriticalSection( FCS );
	end;
end;

procedure TOggVorbisEncoder.WriteOldestBodyAndFreeToStream( const Stream: TStream);
var
	pd: TOggVorbisEncoder_PacketData;
begin
	EnterCriticalSection( FCS );
	try
		pd := TOggVorbisEncoder_PacketData( FDestChain.First );
		if not Assigned( pd ) then Exit;
		pd.Extract();
	finally
		LeaveCriticalSection( FCS );
	end;
	Stream.WriteBuffer( pd.Data^, pd.Length );
	pd.Free;
end;

{ TDataEncodeThread }

constructor TDataEncodeThread.Create( const Parent: TOggVorbisEncoder );
begin
	inherited Create( True );
	FParent := Parent;
end;

destructor TDataEncodeThread.Destroy;
begin
	inherited;
end;

procedure TDataEncodeThread.Execute;
begin
	while not Terminated do
		if ( not FParent.Encode() )and( not Terminated ) then Suspend(); //鎖Ȃ͋x

	//IOɏȂƂȂȂ̂Ă܂
	while FParent.Encode() do ;
end;

initialization
	IsMultiThread := True;
	try
		ovlib := TOggVorbisEncodeFunctions.Create();
	except
		ovlib := nil;
	end;

finalization
	if Assigned( ovlib ) then ovlib.Free;

end.
