unit OggVorbisFileHeader;

interface

uses
	Windows, SysUtils, OggVorbisHeader;

{$A8}

const
	NOTOPEN  = 0;
	PARTOPEN = 1;
	OPENED   = 2;
	STREAMSET= 3;
	INITSET  = 4;
	OV_FALSE = -1;
	OV_EOF   = -2;
	OV_HOLE  = -3;
	OV_EREAD = -128;
	OV_EFAULT= -129;
	OV_EIMPL = -130;
	OV_EINVAL= -131;
	OV_ENOTVORBIS = -132;
	OV_EBADHEADER = -133;
	OV_EVERSION   = -134;
	OV_ENOTAUDIO  = -135;
	OV_EBADPACKET = -136;
	OV_EBADLINK   = -137;
	OV_ENOSEEK    = -138;

type
	POggVorbisFile = ^TOggVorbisFile;
	TOggVorbisFile = record
		Data: array[0..704+256-1] of Char;
	end;

	TOVCallbacks_ReadFunc = function ( const Dest: Pointer; const Size, nMemBlocks: Cardinal; const Source: Pointer ): Cardinal;
	TOVCallbacks_SeekFunc = function ( const Source: Pointer; const Offset: Int64; const Whence: Integer ): Integer;
	TOVCallbacks_CloseFunc = function ( const Source: Pointer ): Integer;
	TOVCallbacks_TellFunc = function ( const Source: Pointer ): Longint;

	POVCallbacks = ^TOVCallbacks;
	TOVCallbacks = record
		ReadFunc: TOVCallbacks_ReadFunc;
		SeekFunc: TOVCallbacks_SeekFunc;
		CloseFunc: TOVCallbacks_CloseFunc;
		TellFunc: TOVCallbacks_TellFunc;
	end;

	TOVF_ov_clear_func =
		function ( const vf: POggVorbisFile ): Integer; cdecl;

	TOVF_ov_open_callbacks_func =
		function ( const DataSource: Pointer; const vf: POggVorbisFile; const Initial: PChar; const iBytes: Longint; const Callbacks: TOVCallbacks): Integer; cdecl;

	TOVF_ov_test_callbacks_func =
		function ( const DataSource: Pointer; const vf: POggVorbisFile; const Initial: PChar; const iBytes: Longint; const Callbacks: TOVCallbacks): Integer; cdecl;

	TOVF_ov_test_open_func =
		function ( const vf: POggVorbisFile ): Integer; cdecl;

	TOVF_ov_bitrate_func =
		function ( const vf: POggVorbisFile; const i: Integer ): Longint; cdecl;

	TOVF_ov_bitrate_instant_func =
		function ( const vf: POggVorbisFile ): Longint; cdecl;

	TOVF_ov_streams_func =
		function ( const vf: POggVorbisFile ): Longint; cdecl;

	TOVF_ov_seekable_func =
		function ( const vf: POggVorbisFile ): Longint; cdecl;

	TOVF_ov_serialnumber_func =
		function ( const vf: POggVorbisFile; const i: Integer ): Longint; cdecl;

	TOVF_ov_raw_total_func =
		function ( const vf: POggVorbisFile; const i: Integer ): Int64; cdecl;

	TOVF_ov_pcm_total_func =
		function ( const vf: POggVorbisFile; const i: Integer ): Int64; cdecl;

	TOVF_ov_time_total_func =
		function ( const vf: POggVorbisFile; const i: Integer ): Double; cdecl;

	TOVF_ov_raw_seek_func =
		function ( const vf: POggVorbisFile; const Pos: Int64 ): Integer; cdecl;

	TOVF_ov_pcm_seek_func =
		function ( const vf: POggVorbisFile; const Pos: Int64 ): Integer; cdecl;

	TOVF_ov_pcm_seek_page_func =
		function ( const vf: POggVorbisFile; const Pos: Int64 ): Integer; cdecl;

	TOVF_ov_time_seek_func =
		function ( const vf: POggVorbisFile; const Pos: Double ): Integer; cdecl;

	TOVF_ov_time_seek_page_func =
		function ( const vf: POggVorbisFile; const Pos: Double ): Integer; cdecl;

	TOVF_ov_raw_seek_lap_func =
		function ( const vf: POggVorbisFile; const Pos: Int64 ): Integer; cdecl;

	TOVF_ov_pcm_seek_lap_func =
		function ( const vf: POggVorbisFile; const Pos: Int64 ): Integer; cdecl;

	TOVF_ov_pcm_seek_page_lap_func =
		function ( const vf: POggVorbisFile; const Pos: Int64): Integer; cdecl;

	TOVF_ov_time_seek_lap_func =
		function ( const vf: POggVorbisFile; const Pos: Double ): Integer; cdecl;

	TOVF_ov_time_seek_page_lap_func =
		function ( const vf: POggVorbisFile; const Pos: Double ): Integer; cdecl;

	TOVF_ov_raw_tell_func =
		function ( const vf: POggVorbisFile ): Int64; cdecl;

	TOVF_ov_pcm_tell_func =
		function ( const vf: POggVorbisFile ): Int64; cdecl;

	TOVF_ov_time_tell_func =
		function ( const vf: POggVorbisFile ): Double; cdecl;

	TOVF_ov_info_func =
		function ( const vf: POggVorbisFile; const Link: Integer ): PVorbisInfo; cdecl;

	TOVF_ov_comment_func =
		function ( const vf: POggVorbisFile; const Link: Integer ): PVorbisComment; cdecl;

	TOVF_ov_read_func =
		function ( const vf: POggVorbisFile; const Buffer: PChar; const Length: Integer; const bigendianp: Integer; const Word: Integer; const sgned: Integer; const bitstream: PInteger ): LongInt; cdecl;

	TOVF_ov_crosslap_func =
		function ( const vf1, vf2 : POggVorbisFile ): Integer; cdecl;

	TOggVorbisFileFunctions = class
	private
		Fov_clear: TOVF_ov_clear_func;
		Fov_open_callbacks: TOVF_ov_open_callbacks_func;
		Fov_test_callbacks: TOVF_ov_test_callbacks_func;
		Fov_test_open: TOVF_ov_test_open_func;
		Fov_bitrate: TOVF_ov_bitrate_func;
		Fov_bitrate_instant: TOVF_ov_bitrate_instant_func;
		Fov_streams: TOVF_ov_streams_func;
		Fov_seekable: TOVF_ov_seekable_func;
		Fov_serialnumber: TOVF_ov_serialnumber_func;
		Fov_raw_total: TOVF_ov_raw_total_func;
		Fov_pcm_total: TOVF_ov_pcm_total_func;
		Fov_time_total: TOVF_ov_time_total_func;
		Fov_raw_seek: TOVF_ov_raw_seek_func;
		Fov_pcm_seek: TOVF_ov_pcm_seek_func;
		Fov_pcm_seek_page: TOVF_ov_pcm_seek_page_func;
		Fov_time_seek: TOVF_ov_time_seek_func;
		Fov_time_seek_page: TOVF_ov_time_seek_page_func;
		Fov_raw_seek_lap: TOVF_ov_raw_seek_lap_func;
		Fov_pcm_seek_lap: TOVF_ov_pcm_seek_lap_func;
		Fov_pcm_seek_page_lap: TOVF_ov_pcm_seek_page_lap_func;
		Fov_time_seek_lap: TOVF_ov_time_seek_lap_func;
		Fov_time_seek_page_lap: TOVF_ov_time_seek_page_lap_func;
		Fov_raw_tell: TOVF_ov_raw_tell_func;
		Fov_pcm_tell: TOVF_ov_pcm_tell_func;
		Fov_time_tell: TOVF_ov_time_tell_func;
		Fov_info: TOVF_ov_info_func;
		Fov_comment: TOVF_ov_comment_func;
		Fov_read: TOVF_ov_read_func;
		Fov_crosslap: TOVF_ov_crosslap_func;
	protected
		FVORBISFILEDLLHandle: THandle;
		procedure CallFuncBefore();
		procedure CallFuncAfter();
		constructor Create( const DLLName: string ); overload;
	public
		constructor Create(); overload;
		destructor Destroy(); override;

		function ov_clear( const vf: POggVorbisFile ): Integer;
		function ov_open_callbacks( const DataSource: Pointer; const vf: POggVorbisFile; const Initial: PChar; const iBytes: Longint; const Callbacks: TOVCallbacks): Integer;
		function ov_test_callbacks( const DataSource: Pointer; const vf: POggVorbisFile; const Initial: PChar; const iBytes: Longint; const Callbacks: TOVCallbacks): Integer;
		function ov_test_open( const vf: POggVorbisFile ): Integer;
		function ov_bitrate( const vf: POggVorbisFile; const i: Integer ): Longint;
		function ov_bitrate_instant( const vf: POggVorbisFile ): Longint;
		function ov_streams( const vf: POggVorbisFile ): Longint;
		function ov_seekable( const vf: POggVorbisFile ): Longint;
		function ov_serialnumber( const vf: POggVorbisFile; const i: Integer ): Longint;
		function ov_raw_total( const vf: POggVorbisFile; const i: Integer ): Int64;
		function ov_pcm_total( const vf: POggVorbisFile; const i: Integer ): Int64;
		function ov_time_total( const vf: POggVorbisFile; const i: Integer ): Double;
		function ov_raw_seek( const vf: POggVorbisFile; const Pos: Int64 ): Integer;
		function ov_pcm_seek( const vf: POggVorbisFile; const Pos: Int64 ): Integer;
		function ov_pcm_seek_page( const vf: POggVorbisFile; const Pos: Int64 ): Integer;
		function ov_time_seek( const vf: POggVorbisFile; const Pos: Double ): Integer;
		function ov_time_seek_page( const vf: POggVorbisFile; const Pos: Double ): Integer;
		function ov_raw_seek_lap( const vf: POggVorbisFile; const Pos: Int64 ): Integer;
		function ov_pcm_seek_lap( const vf: POggVorbisFile; const Pos: Int64 ): Integer;
		function ov_pcm_seek_page_lap( const vf: POggVorbisFile; const Pos: Int64): Integer;
		function ov_time_seek_lap( const vf: POggVorbisFile; const Pos: Double ): Integer;
		function ov_time_seek_page_lap( const vf: POggVorbisFile; const Pos: Double ): Integer;
		function ov_raw_tell( const vf: POggVorbisFile ): Int64;
		function ov_pcm_tell( const vf: POggVorbisFile ): Int64;
		function ov_time_tell( const vf: POggVorbisFile ): Double;
		function ov_info( const vf: POggVorbisFile; const Link: Integer ): PVorbisInfo;
		function ov_comment( const vf: POggVorbisFile; const Link: Integer ): PVorbisComment;
		function ov_read( const vf: POggVorbisFile; const Buffer: PChar; const Length: Integer; const bigendianp: Integer; const Word: Integer; const sgned: Integer; const bitstream: PInteger ): LongInt;
		function ov_crosslap( const vf1, vf2 : POggVorbisFile ): Integer;
	end;


implementation

const
	VORBISFILE_DLL_NAME = 'vorbisfile.dll';

{ TOggVorbisFileFunctions }

procedure TOggVorbisFileFunctions.CallFuncAfter;
begin
	Set8087CW(Default8087CW);
end;

procedure TOggVorbisFileFunctions.CallFuncBefore;
begin
	Set8087CW($133f);
end;

constructor TOggVorbisFileFunctions.Create(const DLLName: string);
var
	s: string;
begin
	if DLLName = '' then
		s := VORBISFILE_DLL_NAME
	else
		s := DLLName;

	if not FileExists( s ) then
		raise Exception.Create( s+' ܂B' );

	FVORBISFILEDLLHandle := SafeLoadLibrary( s );

	if FVORBISFILEDLLHandle = 0 then
		raise Exception.Create( s+' ̓ǂݍ݂Ɏs܂B' );

	inherited Create();

	Fov_clear := GetProcAddress( FVORBISFILEDLLHandle, 'ov_clear' );
	Fov_open_callbacks := GetProcAddress( FVORBISFILEDLLHandle, 'ov_open_callbacks' );
	Fov_test_callbacks := GetProcAddress( FVORBISFILEDLLHandle, 'ov_test_callbacks' );
	Fov_test_open := GetProcAddress( FVORBISFILEDLLHandle, 'ov_test_open' );
	Fov_bitrate := GetProcAddress( FVORBISFILEDLLHandle, 'ov_bitrate' );
	Fov_bitrate_instant := GetProcAddress( FVORBISFILEDLLHandle, 'ov_bitrate_instant' );
	Fov_streams := GetProcAddress( FVORBISFILEDLLHandle, 'ov_streams' );
	Fov_seekable := GetProcAddress( FVORBISFILEDLLHandle, 'ov_seekable' );
	Fov_serialnumber := GetProcAddress( FVORBISFILEDLLHandle, 'ov_serialnumber' );
	Fov_raw_total := GetProcAddress( FVORBISFILEDLLHandle, 'ov_raw_total' );
	Fov_pcm_total := GetProcAddress( FVORBISFILEDLLHandle, 'ov_pcm_total' );
	Fov_time_total := GetProcAddress( FVORBISFILEDLLHandle, 'ov_time_total' );
	Fov_raw_seek := GetProcAddress( FVORBISFILEDLLHandle, 'ov_raw_seek' );
	Fov_pcm_seek := GetProcAddress( FVORBISFILEDLLHandle, 'ov_pcm_seek' );
	Fov_pcm_seek_page := GetProcAddress( FVORBISFILEDLLHandle, 'ov_pcm_seek_page' );
	Fov_time_seek := GetProcAddress( FVORBISFILEDLLHandle, 'ov_time_seek' );
	Fov_time_seek_page := GetProcAddress( FVORBISFILEDLLHandle, 'ov_time_seek_page' );
	Fov_raw_seek_lap := GetProcAddress( FVORBISFILEDLLHandle, 'ov_raw_seek_lap' );
	Fov_pcm_seek_lap := GetProcAddress( FVORBISFILEDLLHandle, 'ov_pcm_seek_lap' );
	Fov_pcm_seek_page_lap := GetProcAddress( FVORBISFILEDLLHandle, 'ov_pcm_seek_page_lap' );
	Fov_time_seek_lap := GetProcAddress( FVORBISFILEDLLHandle, 'ov_time_seek_lap' );
	Fov_time_seek_page_lap := GetProcAddress( FVORBISFILEDLLHandle, 'ov_time_seek_page_lap' );
	Fov_raw_tell := GetProcAddress( FVORBISFILEDLLHandle, 'ov_raw_tell' );
	Fov_pcm_tell := GetProcAddress( FVORBISFILEDLLHandle, 'ov_pcm_tell' );
	Fov_time_tell := GetProcAddress( FVORBISFILEDLLHandle, 'ov_time_tell' );
	Fov_info := GetProcAddress( FVORBISFILEDLLHandle, 'ov_info' );
	Fov_comment := GetProcAddress( FVORBISFILEDLLHandle, 'ov_comment' );
	Fov_read := GetProcAddress( FVORBISFILEDLLHandle, 'ov_read' );
	Fov_crosslap := GetProcAddress( FVORBISFILEDLLHandle, 'ov_crosslap' );

end;

constructor TOggVorbisFileFunctions.Create;
begin
	Create( VORBISFILE_DLL_NAME );
end;

destructor TOggVorbisFileFunctions.Destroy;
begin
	if FVORBISFILEDLLHandle <> 0 then FreeLibrary( FVORBISFILEDLLHandle );

	inherited;
end;

function TOggVorbisFileFunctions.ov_bitrate(const vf: POggVorbisFile;
  const i: Integer): Longint;
begin
	CallFuncBefore();
	Result := Fov_bitrate( vf, i );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_bitrate_instant(
  const vf: POggVorbisFile): Longint;
begin
	CallFuncBefore();
	Result := Fov_bitrate_instant( vf );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_clear(
  const vf: POggVorbisFile): Integer;
begin
	CallFuncBefore();
	Result := Fov_clear( vf );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_comment(const vf: POggVorbisFile;
  const Link: Integer): PVorbisComment;
begin
	CallFuncBefore();
	Result := Fov_comment( vf, Link );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_crosslap(const vf1,
  vf2: POggVorbisFile): Integer;
begin
	CallFuncBefore();
	Result := Fov_crosslap( vf1, vf2 );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_info(const vf: POggVorbisFile;
  const Link: Integer): PVorbisInfo;
begin
	CallFuncBefore();
	Result := Fov_info( vf, Link );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_open_callbacks(
  const DataSource: Pointer; const vf: POggVorbisFile;
  const Initial: PChar; const iBytes: Integer;
  const Callbacks: TOVCallbacks): Integer;
begin
	CallFuncBefore();
	Result := Fov_open_callbacks( DataSource, vf, Initial, iBytes, Callbacks );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_pcm_seek(const vf: POggVorbisFile;
  const Pos: Int64): Integer;
begin
	CallFuncBefore();
	Result := Fov_pcm_seek( vf, Pos );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_pcm_seek_lap(const vf: POggVorbisFile;
  const Pos: Int64): Integer;
begin
	CallFuncBefore();
	Result := Fov_pcm_seek_lap( vf, Pos );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_pcm_seek_page(const vf: POggVorbisFile;
  const Pos: Int64): Integer;
begin
	CallFuncBefore();
	Result := Fov_pcm_seek_page( vf, Pos );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_pcm_seek_page_lap(
  const vf: POggVorbisFile; const Pos: Int64): Integer;
begin
	CallFuncBefore();
	Result := Fov_pcm_seek_page_lap( vf, Pos );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_pcm_tell(
  const vf: POggVorbisFile): Int64;
begin
	CallFuncBefore();
	Result := Fov_pcm_tell( vf );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_pcm_total(const vf: POggVorbisFile;
  const i: Integer): Int64;
begin
	CallFuncBefore();
	Result := Fov_pcm_total( vf, i );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_raw_seek(const vf: POggVorbisFile;
  const Pos: Int64): Integer;
begin
	CallFuncBefore();
	Result := Fov_raw_seek( vf, Pos );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_raw_seek_lap(const vf: POggVorbisFile;
  const Pos: Int64): Integer;
begin
	CallFuncBefore();
	Result := Fov_raw_seek_lap( vf, Pos );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_raw_tell(
  const vf: POggVorbisFile): Int64;
begin
	CallFuncBefore();
	Result := Fov_raw_tell( vf );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_raw_total(const vf: POggVorbisFile;
  const i: Integer): Int64;
begin
	CallFuncBefore();
	Result := Fov_raw_total( vf, i );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_read(const vf: POggVorbisFile;
  const Buffer: PChar; const Length, bigendianp, Word, sgned: Integer;
  const bitstream: PInteger): LongInt;
begin
	CallFuncBefore();
	Result := Fov_read( vf, Buffer, Length, bigendianp, Word, sgned, bitstream );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_seekable(
  const vf: POggVorbisFile): Longint;
begin
	CallFuncBefore();
	Result := Fov_seekable( vf );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_serialnumber(const vf: POggVorbisFile;
  const i: Integer): Longint;
begin
	CallFuncBefore();
	Result := Fov_serialnumber( vf, i );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_streams(
  const vf: POggVorbisFile): Longint;
begin
	CallFuncBefore();
	Result := Fov_streams( vf );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_test_callbacks(
  const DataSource: Pointer; const vf: POggVorbisFile;
  const Initial: PChar; const iBytes: Integer;
  const Callbacks: TOVCallbacks): Integer;
begin
	CallFuncBefore();
	Result := Fov_test_callbacks( DataSource, vf, Initial, iBytes, Callbacks );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_test_open(
  const vf: POggVorbisFile): Integer;
begin
	CallFuncBefore();
	Result := Fov_test_open( vf );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_time_seek(const vf: POggVorbisFile;
  const Pos: Double): Integer;
begin
	CallFuncBefore();
	Result := Fov_time_seek( vf, Pos );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_time_seek_lap(const vf: POggVorbisFile;
  const Pos: Double): Integer;
begin
	CallFuncBefore();
	Result := Fov_time_seek_lap( vf, Pos );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_time_seek_page(
  const vf: POggVorbisFile; const Pos: Double): Integer;
begin
	CallFuncBefore();
	Result := Fov_time_seek_page( vf, Pos );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_time_seek_page_lap(
  const vf: POggVorbisFile; const Pos: Double): Integer;
begin
	CallFuncBefore();
	Result := Fov_time_seek_page_lap( vf, Pos );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_time_tell(
  const vf: POggVorbisFile): Double;
begin
	CallFuncBefore();
	Result := Fov_time_tell( vf );
	CallFuncAfter();
end;

function TOggVorbisFileFunctions.ov_time_total(const vf: POggVorbisFile;
  const i: Integer): Double;
begin
	CallFuncBefore();
	Result := Fov_time_total( vf, i );
	CallFuncAfter();
end;

end.
