unit WuVorbisHeader;

interface

uses
	Windows, OggVorbisHeader, OggVorbisFileHeader;

const
	WU_CPU_HAS_FPU  = $000010000;
	WU_CPU_HAS_MMX  = $000020000;
	WU_CPU_HAS_3DN  = $000040000;
	WU_CPU_HAS_SSE  = $000080000;
	WU_CPU_HAS_CMOV = $000100000;
	WU_CPU_HAS_E3DN = $000200000;
	WU_CPU_HAS_EMMX = $000400000;
	WU_CPU_HAS_SSE2 = $000800000;
	WU_CPU_HAS_TSC  = $001000000;
	WU_CPU_FEATURE_MASK = $0ffff0000;
	WU_CPU_IS_INTEL = $000000010;
	WU_CPU_IS_AMD   = $000000020;
	WU_CPU_IS_IDT   = $000000030;
	WU_CPU_IS_CYRIX = $000000040;
	WU_CPU_IS_NEXGEN = $000000050;
	WU_CPU_IS_RISE  = $000000060;
	WU_CPU_IS_UMC   = $000000070;
	WU_CPU_IS_TRANSMETA = $000000080;
	WU_CPU_IS_UNKNOWN = $000000000;
	WU_CPU_VENDOR_MASK = $000000ff0;
	WU_CPU_FAMILY_MASK = $00000000f;

type
	TWU_Query_sizeof_OggVorbis_File_func =
		function (): Integer; stdcall;

	TWU_ov_read_float_func =
		function ( const vf: POggVorbisFile; const PCMChannels: PPSingleArrayArray; const Sample: Integer; const bitstream: PInteger ): Longint; cdecl;

	TWU_wu_SetCPUType_func =
		procedure ( const CPUType: DWORD ); cdecl;

	TWU_wu_DetectCPU_func =
		function (): Integer; cdecl;

	TWU_wu_ScaleOutput_func =
		procedure ( const Scale: Single ); cdecl;

	TWuOggVorbisFileFunctions = class( 	TOggVorbisFileFunctions )
	private
		FQuery_sizeof_OggVorbis_File: TWU_Query_sizeof_OggVorbis_File_func;
		Fov_read_float: TWU_ov_read_float_func;
		Fwu_SetCPUType: TWU_wu_SetCPUType_func;
		Fwu_DetectCPU: TWU_wu_DetectCPU_func;
		Fwu_ScaleOutput: TWU_wu_ScaleOutput_func;
	public
		constructor Create();
		function ov_read_float( const vf: POggVorbisFile; const PCMChannels: PPSingleArrayArray; const Sample: Integer; const bitstream: PInteger ): Longint;
		procedure ScaleOutput( const Scale: Single );
	end;

implementation

const
	WUVORBIS_DLL_NAME = 'wuvorbis.dll';
{ TWuOggVorbisFileFunctions }

constructor TWuOggVorbisFileFunctions.Create;
begin
	inherited Create( WUVORBIS_DLL_NAME );

	FQuery_sizeof_OggVorbis_File := GetProcAddress( FVORBISFILEDLLHandle, 'Query_sizeof_OggVorbis_File' );
	Fov_read_float := GetProcAddress( FVORBISFILEDLLHandle, 'ov_read_float' );
	Fwu_SetCPUType := GetProcAddress( FVORBISFILEDLLHandle, 'wu_SetCPUType' );
	Fwu_DetectCPU := GetProcAddress( FVORBISFILEDLLHandle, 'wu_DetectCPU' );
	Fwu_ScaleOutput := GetProcAddress( FVORBISFILEDLLHandle, 'wu_ScaleOutput' );

	Fwu_SetCPUType( Fwu_DetectCPU() );
end;

function TWuOggVorbisFileFunctions.ov_read_float(const vf: POggVorbisFile;
  const PCMChannels: PPSingleArrayArray; const Sample: Integer;
  const bitstream: PInteger): Longint;
begin
	CallFuncBefore();
	Result := Fov_read_float( vf, PCMChannels, Sample, bitstream );
	CallFuncAfter();
end;

procedure TWuOggVorbisFileFunctions.ScaleOutput(const Scale: Single);
begin
	CallFuncBefore();
	Fwu_ScaleOutput( Scale );
	CallFuncAfter();
end;

end.
