unit uChainList;

interface

type
	TChainEntry = class;
	TChainList = class;

	TChainEntry = class( TObject )
	private
		FParent: TChainList;
		FNext: TChainEntry;
		FPrev: TChainEntry;
	public
		constructor Create();
		destructor Destroy(); override;
		procedure Extract();
		property Parent: TChainList read FParent;
		property Next: TChainEntry read FNext;
		property Prev: TChainEntry read FPrev;
	end;

	TChainList = class( TObject )
	private
		FCount: Cardinal;
		FFirst: TChainEntry;
		FLast: TChainEntry;
	protected
		procedure EnteryAdded( const Item: TChainEntry ); virtual;
		procedure EnteryExtract( const Item: TChainEntry ); virtual;
	public
		constructor Create();
		destructor Destroy(); override;
		procedure Clear();
		procedure Extract( const Item: TChainEntry );
		procedure InsertFirst( const Item: TChainEntry );
		procedure InsertLast( const Item: TChainEntry );
		procedure InsertBefore( const Item: TChainEntry; const A: TChainEntry );
		procedure InsertAfter( const Item: TChainEntry; const A: TChainEntry );
		property Count: Cardinal read FCount;
		property First: TChainEntry read FFirst;
		property Last: TChainEntry read FLast;
	end;

implementation

{ TChainEntry }

constructor TChainEntry.Create;
begin
	inherited;
	FParent := nil;
	FNext := nil;
	FPrev := nil;
end;

destructor TChainEntry.Destroy;
begin
	if Assigned( FParent ) then FParent.Extract( Self );

	inherited;
end;

procedure TChainEntry.Extract;
begin
	if Assigned( FParent ) then FParent.Extract( Self );
end;

{ TChainList }

procedure TChainList.Clear;
begin
	while Assigned( FFirst ) do FFirst.Free;
end;

constructor TChainList.Create;
begin
	inherited;

	FCount := 0;
	FFirst := nil;
	FLast := nil;
end;

destructor TChainList.Destroy;
begin
	while Assigned( FFirst ) do FFirst.Free;
	inherited;
end;

procedure TChainList.EnteryAdded(const Item: TChainEntry);
begin
//ǉꂽɌĂ΂
end;

procedure TChainList.EnteryExtract(const Item: TChainEntry);
begin
//폜OɌĂ΂
end;

procedure TChainList.Extract(const Item: TChainEntry);
begin
	if Item.FParent = Self then begin
		EnteryExtract( Item );
		if ( not Assigned( Item.FPrev ) )and( not Assigned( Item.FNext ) ) then begin
			FFirst := nil;
			FLast := nil;
		end else begin
			if FFirst = Item then FFirst := Item.FNext;
			if FLast = Item then FLast := Item.FPrev;
			if Assigned( Item.FNext ) then Item.FNext.FPrev := Item.FPrev;
			if Assigned( Item.FPrev ) then Item.FPrev.FNext := Item.FNext;
		end;
		Item.FParent := nil;
		Dec( FCount );
	end;
end;

procedure TChainList.InsertAfter(const Item, A: TChainEntry);
begin
	if A.FParent = Self then begin
		if A = FLast then
			InsertLast( Item )
		else begin
			Item.FNext := A.FNext;
			Item.FPrev := A;
			A.FNext.FPrev := Item;
			A.FNext := Item;
			Item.FParent := Self;
			Inc( FCount );
		end;
		EnteryAdded( Item );
	end;
end;

procedure TChainList.InsertBefore(const Item, A: TChainEntry);
begin
	if A.FParent = Self then begin
		if A = FFirst then
			InsertFirst( Item )
		else begin
			Item.FPrev := A.FPrev;
			Item.FNext := A;
			A.FPrev.FNext := Item;
			A.FPrev := Item;
			Item.FParent := Self;
			Inc( FCount );
		end;
		EnteryAdded( Item );
	end;
end;

procedure TChainList.InsertFirst(const Item: TChainEntry);
begin
	if Assigned( FFirst ) then begin
		Item.FPrev := nil;
		Item.FNext := FFirst;
		FFirst.FPrev := Item;
		FFirst := Item;
	end else begin
		FFirst := Item;
		FLast := Item;
		Item.FNext := nil;
		Item.FPrev := nil;
	end;
	Item.FParent := Self;
	Inc( FCount );
	EnteryAdded( Item );
end;

procedure TChainList.InsertLast(const Item: TChainEntry);
begin
	if Assigned( FLast ) then begin
		Item.FNext := nil;
		Item.FPrev := FLast;
		FLast.FNext := Item;
		FLast := Item;
	end else begin
		FFirst := Item;
		FLast := Item;
		Item.FNext := nil;
		Item.FPrev := nil;
	end;
	Item.FParent := Self;
	Inc( FCount );
	EnteryAdded( Item );
end;

end.
